using System;
using System.Collections;
using System.ComponentModel;


namespace ITypedListDataSourceSample
{
	/// <summary>
	/// Summary description for DataSourceArrayList.
	/// </summary>
	public class DataSourceArrayList : ArrayList, ITypedList
	{
		
		public Type[] ItemTypeList;

		public PropertyDescriptorCollection pdc;

		public DataSourceArrayList(Type[] list, string[] propertyNames) : base()
		{
			ItemTypeList = list;
			int nTypes = list.GetLength(0);
			MyPropertyDescriptor[] pds = new MyPropertyDescriptor[nTypes];
			for(int i = 0; i < nTypes; ++i)
			{
				PropertyDescriptor pd = TypeDescriptor.CreateProperty(typeof(MyObject), propertyNames[i], list[i], null );
				pds[i] = new MyPropertyDescriptor(pd, this, i);
					
			}
			pdc = new PropertyDescriptorCollection(pds);
			 
		}

		public PropertyDescriptorCollection GetItemProperties(PropertyDescriptor[]
			listAccessors)
		{
			return this.pdc;
		}

		public string GetListName(PropertyDescriptor[] listAccessors)
		{

			return "TypedObject";

		} 
	}

	class MyPropertyDescriptor : PropertyDescriptor
	{
		DataSourceArrayList list;
		PropertyDescriptor basePropDesc;
		private int id = -1;
				
		public MyPropertyDescriptor(PropertyDescriptor basePropDesc, DataSourceArrayList list, int id)
			:base(basePropDesc)
		{
			this.basePropDesc = basePropDesc;
			this.id = id;
			this.list = list;
		}

		/// <override/>
		public override bool CanResetValue(object component)
		{
			return this.basePropDesc.CanResetValue(component);
		}

		/// <override/>
		public override object GetValue(object component)
		{
			try
			{
				MyObject o = component as MyObject;
				if(o == null)
					return null;
				return o.list[id];
			}
			catch
			{
				throw new ArgumentException("component must be MyObject type");
			}
			
			 
		}

		/// <override/>
		public override void ResetValue(object component)
		{
			SetValue(component, null);
		}

		/// <override/>
		public override void SetValue(object component, object value)
		{

			try
			{
				MyObject o = component as MyObject;
				if(o == null)
					return;
				o.list[id] = value;

			}
			catch
			{
				throw new ArgumentException("component must be MyObject type");
			}
		}

		/// <override/>
		public override bool ShouldSerializeValue(object component)
		{
			return this.basePropDesc.ShouldSerializeValue(component);
		}

		/// <override/>
		public override Type ComponentType 
		{ 
			get
			{
				return this.basePropDesc.ComponentType;
			}
		}
			
		/// <override/>
		public override bool IsReadOnly 
		{ 
			get
			{
				return false;
			}
		}
			
		/// <override/>
		public override Type PropertyType 
		{ 
			get
			{
				return this.basePropDesc.PropertyType;
			}
		}
	}

	public class MyObject
	{
		internal object[] list;
		public MyObject(int nItems)
		{
			list = new object[nItems];
		}

	}

}
